#include <plib.h>
#include <p32xxxx.h>

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_1
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4

#define SystemFrequency		60000000L
#define CoreTick		( SystemFrequency / 2 / TogglesPerSec )
#define TogglesPerSec		10

void __ISR(_CORE_TIMER_VECTOR, ipl3) ISR_CoreTimer(void)
{
    // 清除Core Timer中斷旗標。
    mCTClearIntFlag();

    // To Do : 更新Core Timer之Period。
    UpdateCoreTimer(CoreTick);

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

int main(void)
{
    // Disable the JTAG function
    DDPCONbits.JTAGEN = 0;

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    INTEnableSystemMultiVectoredInt();

    // 使用C32 Core Timer Function啟用Core Timer，並指定其中斷優先權。
    mConfigIntCoreTimer((CT_INT_ON | CT_INT_PRIOR_3));

    // 使用C32 Core Timer Function進行設定。
    OpenCoreTimer(CoreTick);

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    while (1);
}
