#include <plib.h>
#include <p32xxxx.h>

/* Configuration Settings */
// OSC Input = HS w/PLL
// CPU Clock = ( ( OSC Input / 2 ) * 15 ) / 1
// PB Clock = CPU Clock / 1
// Disabld WDT , ICSP/ICD = PGC4 , PGD4
#pragma config FNOSC = PRIPLL , POSCMOD = HS
#pragma config FPLLIDIV = DIV_2 , FPLLMUL = MUL_15 , FPLLODIV = DIV_1
#pragma config FPBDIV = DIV_1
#pragma config FWDTEN = OFF , ICESEL = ICS_PGx4

#define SystemFrequency		60000000L
#define PBFrequency		60000000L
#define Timer1Tick		( PBFrequency / 256 / TogglesPerSec )
#define TogglesPerSec		10

void __ISR(_TIMER_1_VECTOR, ipl4) ISR_Timer1(void)
{
    // 清除Timer中斷旗標。
    mT1ClearIntFlag();

    // LED狀態反向。
    mPORTAToggleBits(BIT_9);
    mPORTCToggleBits(BIT_3);
}

int main(void)
{
    // Disable the JTAG function
    DDPCONbits.JTAGEN = 0;

    // 使用C32的SYSTEM Config巨集將CPU的操作最佳化，但不修改到FPBDIV的設定。
    SYSTEMConfig(SystemFrequency, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);
    // SYSTEMConfigPerformance( SystemFrequency  );
    // SYSTEMConfigWaitStates( SystemFrequency );

    INTEnableSystemMultiVectoredInt();

    // 使用C32 Timer Function啟用Timer，並指定其中斷優先權。
    ConfigIntTimer1(T1_INT_ON | T1_INT_PRIOR_4);

    // 使用C32 Timer Function進行設定。
    OpenTimer1(T1_ON | T1_IDLE_CON | T1_GATE_OFF | T1_PS_1_256 | T1_SOURCE_INT, Timer1Tick);

    // LED對應之I/O Port設定成數位輸出，並指定I/O Port狀態。
    mPORTASetPinsDigitalOut(BIT_9);
    mPORTCSetPinsDigitalOut(BIT_3);
    mPORTAClearBits(BIT_9);
    mPORTCSetBits(BIT_3);

    while (1);
}
